package org.openscm.kundo.factory

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.common.logging.SimpleLogger
import org.openscm.kundo.strategy.ModuleStrategy
import org.openscm.kundo.strategy.DefaultModuleStrategy
import org.openscm.kundo.strategy.MavenModuleStrategy
import org.openscm.kundo.strategy.EclipseModuleStrategy
import org.openscm.kundo.strategy.KundoModuleStrategy

/**
 * Module Strategy Factory
 * @author <a href="mailto:nigel.garner@upco.co.uk">Nigel Garner</a>
 * @version 1.0.0
 * 
 * Description: The Module Strategy Factory identifies and returns an appropriate Module Strategy based on the build context. 
 * 
 * <p>The Module Strategy Factory identifies an appropriate module strategy from from data within the BuildContext. 
 * If a pom file is available in the build context pom.location property then a MavenModuleStrategy will be returned. 
 * If no strategy can be identified then the DefaultModuleStrategy will be returned</p>
 */
public class ModuleStrategyFactory{
	
	/** AntBuilder instance. **/
	private AntBuilder ant
	
	/** Logger instance. **/
	final SimpleLogger log
	
	/**
	 * Constructor sets ant instance
	 * @param ant			AntBuilder instance
	 */
	ModuleStrategyFactory( ant ){
		this.ant = ant
		log = SimpleLogger.getInstance( ant )
	}

	/**
	 * Identifies and returns an appropriate ModuleStrategy for the current state. 
	 * @param buildContext	The current BuildContext instance containing the global ant project.
	 * @return ModuleStrategy	Instance of the appropriate ModuleStrategy for the current build context data.
	 */
	ModuleStrategy getModuleStrategy( buildContext ) {
		 
		def strategy 
		if( buildContext.isSet( "helper.strategy" ) ){
			strategy = buildContext.get( "helper.strategy" )
		}

		// ---------------------------------------------------
		// Maven Strategy
		// ---------------------------------------------------		
		// Check to see if the pom file exists by retrieving and loading it from the build context.
		if( strategy != null && buildContext.isSet( "pom.location" ) && strategy == "maven" ) {
			def pom = buildContext.get( "pom.location" )
			log.debug( 'Creating Maven Module Strategy' )
			File file = new File( pom )
			if ( file.exists() ) {
				ModuleStrategy ms = new MavenModuleStrategy( ant )
				return ms
			}
		}
		
		// ---------------------------------------------------
		//	Add other strategies here as they are developed... 
		// ---------------------------------------------------
		
		if( strategy != null && strategy.indexOf( "eclipse" ) != -1 ){
			ModuleStrategy ms = new EclipseModuleStrategy( ant )
			return ms
		}
		
		// ---------------------------------------------------
		// Default Strategy		
		// ---------------------------------------------------
		
		if( strategy == null || strategy == "" ){
			log.debug( 'Creating Kundo Module Strategy' )
			ModuleStrategy ms = new KundoModuleStrategy( ant )
			return ms			
		}

		// Last one in the list. Will always fall into this as an empty property is declared
		// in the plugin.properties file.
		def moduleString = ant.project.properties.'reactor.module.list'
		if( moduleString != null ) {
			log.debug( 'Creating Default Module Strategy' )
			ModuleStrategy ms = new DefaultModuleStrategy( ant )
			return ms
		}
	}
}
